/** @license num-to-korean v0.5.3
* https://github.com/huskyhoochu/num-to-korean
* (c) 2020-2024 Seung Hyung Soo
* This source code is licensed under the MIT license found in the
* LICENSE file in the root directory of this source tree.
*/

(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.numToKorean = {}));
})(this, (function (exports) { 'use strict';

    /******************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */

    function __spreadArray(to, from, pack) {
        if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
            if (ar || !(i in from)) {
                if (!ar) ar = Array.prototype.slice.call(from, 0, i);
                ar[i] = from[i];
            }
        }
        return to.concat(ar || Array.prototype.slice.call(from));
    }

    typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
        var e = new Error(message);
        return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
    };

    // 한글로 바꿀 숫자 배열
    var textSymbol = ['', '일', '이', '삼', '사', '오', '육', '칠', '팔', '구'];
    // 4자리마다 반복되는 자릿수 배열
    var powerSymbol = ['', '십', '백', '천'];
    // 4자리마다 커지는 단위수 배열
    var dotSymbol = ['', '만', '억', '조', '경'];

    function splitEvery(n, arr) {
        var result = [];
        var index = 0;
        while (index < arr.length) {
            result.push(arr.slice(index, (index += n)));
        }
        return result;
    }
    function dropLastWhile(pred, arr) {
        var index = arr.length - 1;
        while (index >= 0 && pred(arr[index])) {
            index -= 1;
        }
        return arr.slice(0, index + 1);
    }

    var FormatOptions = {
        NORMAL: 'normal',
        SPACING: 'spacing',
        MIXED: 'mixed',
        LINGUAL: 'lingual',
        LINGUAL_SPACING: 'lingual_spacing',
        FLOAT: 'float', // 소수점 표현
    };
    var NumToKorean = /** @class */ (function () {
        function NumToKorean(num, formatOption) {
            this.originNum = 0;
            this.formatOption = 'normal';
            this.token = [];
            this.result = [];
            this.originNum = NumToKorean.isInteger(num) ? num : 0;
            this.formatOption = formatOption;
        }
        NumToKorean.isInteger = function (num) {
            // eslint-disable-next-line no-restricted-globals
            return isFinite(num) && Math.floor(num) === num;
        };
        NumToKorean.prototype.getToken = function () {
            this.token = this.originNum
                .toString()
                .split('')
                .map(function (numText) { return parseInt(numText, 10); })
                .reverse()
                .map(function (num, index) {
                var powerIndex = index % 4;
                var dotIndex = Math.ceil(index / 4);
                var power = num === 0 ? '' : powerSymbol[powerIndex];
                var dot = powerIndex === 0 ? dotSymbol[dotIndex] : '';
                return {
                    num: num,
                    power: power,
                    dot: dot,
                };
            });
            return this;
        };
        NumToKorean.prototype.flatToken = function (isMixed) {
            this.result = this.token.map(function (item) {
                if (isMixed) {
                    return "".concat(item.num).concat(item.dot);
                }
                var numToText = textSymbol[item.num] || '';
                return "".concat(numToText).concat(item.power).concat(item.dot);
            });
            return this;
        };
        NumToKorean.prototype.flatTokenForFloatMode = function () {
            var dots = [];
            var maxDot = '';
            this.token.forEach(function (item) {
                dots.push(item.dot);
            });
            maxDot = dots.join('').split('').slice(-1)[0];
            if (maxDot === undefined) {
                dots = [];
                this.token.forEach(function (item) {
                    dots.push(item.power);
                });
                maxDot = dots.join('').split('').slice(-1)[0];
                this.result = this.token.map(function (item) { return "".concat(item.num).concat(item.power === maxDot ? ".".concat(item.power) : ''); });
                return this;
            }
            this.result = this.token.map(function (item) { return "".concat(item.num).concat(item.dot === maxDot ? ".".concat(item.dot) : ''); });
            return this;
        };
        NumToKorean.prototype.moveToDotLast = function () {
            var matchResult = this.result.join('').match(/\.(십|백|천|만|억|조|경)/);
            if (matchResult) {
                var exactDot = matchResult[1];
                var removedDotResult = this.result
                    .join('')
                    .replace(/(십|백|천|만|억|조|경)/g, '')
                    .split('');
                this.result = __spreadArray(__spreadArray([], removedDotResult, true), [exactDot], false);
            }
            else {
                this.result = this.result.map(function (item) { return item.replace(/\./g, ''); });
            }
            return this;
        };
        NumToKorean.prototype.removeUnnecessaryChars = function () {
            this.result = this.result
                .join('')
                .replace(/\..*/, function (match) { return match.replace(/,/g, ''); })
                .replace(/\.(?=[십백천만억조경]|$)/, '') // 모든 게 0으로 떨어져서 소수점만 남으면 소수점 제거
                .replace(/\.(\d*[1-9])?(0+)(?=[십백천만억조경])/, function (_, p1) { return (p1 ? ".".concat(p1) : ''); })
                .split('');
            return this;
        };
        NumToKorean.prototype.swapZeroToComma = function () {
            this.result = splitEvery(4, this.result)
                .map(function (item) {
                var droppedZero = dropLastWhile(function (x) { return parseInt(x, 10) === 0; }, item);
                if (droppedZero.length === 4) {
                    droppedZero.splice(3, 0, ',');
                }
                return droppedZero;
            })
                .reduce(function (acc, val) { return acc.concat(val); }, []);
            return this;
        };
        NumToKorean.prototype.onlyAddComma = function () {
            this.result = splitEvery(4, this.result)
                .map(function (item) {
                if (item.length === 4) {
                    item.splice(3, 0, ",");
                }
                return item;
            })
                .reduce(function (acc, val) { return acc.concat(val); }, []);
            return this;
        };
        NumToKorean.prototype.onlyReverse = function () {
            this.result = this.result.reverse();
            return this;
        };
        NumToKorean.prototype.addEmptyPositionWhenSymbolHasValue = function () {
            this.result = splitEvery(4, this.result)
                .map(function (item) {
                return dotSymbol.indexOf(item.join('')) > 0 ? ['', '', '', ''] : item;
            })
                .reduce(function (acc, val) { return acc.concat(val); }, []);
            return this;
        };
        NumToKorean.prototype.addSpacing = function () {
            this.result = this.result
                .filter(function (token) { return token; })
                .map(function (token) {
                if (dotSymbol.indexOf(token.slice(-1)) > 0) {
                    return "".concat(token, " ");
                }
                return token;
            });
            return this;
        };
        NumToKorean.prototype.removeOneStr = function () {
            if (this.result.length === 1) {
                return this;
            }
            var filtered = [];
            for (var i = 0; i < this.result.length; i += 1) {
                // 일의자리의 "일"은 생략되어선 안 된다
                if (i === this.result.length - 1) {
                    filtered.push(this.result[i]);
                    break;
                }
                // 십만자리 이상일 때 만, 억, 조 단위의 "일"은 생략되어선 안 된다
                // 다만 만의자리일 때 만의자리의 "일"은 생략되어야 한다
                if (this.result.length > 5 &&
                    dotSymbol.indexOf(this.result[i].slice(1)) > 0) {
                    filtered.push(this.result[i]);
                    // eslint-disable-next-line no-continue
                    continue;
                }
                // 그 외의 "일"은 생략된다
                if (this.result[i].slice(0, 1) === textSymbol[1]) {
                    filtered.push(this.result[i].slice(1));
                }
                else {
                    filtered.push(this.result[i]);
                }
            }
            this.result = filtered;
            return this;
        };
        NumToKorean.prototype.makeZeroIfResultEmpty = function () {
            if ((this.result.length === 1 && this.result[0] === '') ||
                this.result.length === 0) {
                this.result = ['0'];
            }
            return this;
        };
        NumToKorean.prototype.getResult = function () {
            return this.result;
        };
        NumToKorean.prototype.getNormal = function () {
            return this.getToken()
                .flatToken(false)
                .addEmptyPositionWhenSymbolHasValue()
                .onlyReverse()
                .getResult()
                .join('');
        };
        NumToKorean.prototype.getLingual = function () {
            return this.getToken()
                .flatToken(false)
                .addEmptyPositionWhenSymbolHasValue()
                .onlyReverse()
                .removeOneStr()
                .getResult()
                .join('');
        };
        NumToKorean.prototype.getSpacing = function () {
            return this.getToken()
                .flatToken(false)
                .addEmptyPositionWhenSymbolHasValue()
                .onlyReverse()
                .addSpacing()
                .getResult()
                .join('')
                .trim();
        };
        NumToKorean.prototype.getLingualSpacing = function () {
            return this.getToken()
                .flatToken(false)
                .addEmptyPositionWhenSymbolHasValue()
                .onlyReverse()
                .removeOneStr()
                .addSpacing()
                .getResult()
                .join('')
                .trim();
        };
        NumToKorean.prototype.getMixed = function () {
            return this.getToken()
                .flatToken(true)
                .swapZeroToComma()
                .onlyReverse()
                .addSpacing()
                .makeZeroIfResultEmpty()
                .getResult()
                .join('')
                .trim();
        };
        NumToKorean.prototype.getFloat = function () {
            return this.getToken()
                .flatTokenForFloatMode()
                .onlyAddComma()
                .onlyReverse()
                .moveToDotLast()
                .removeUnnecessaryChars()
                .makeZeroIfResultEmpty()
                .getResult()
                .join('')
                .trim();
        };
        NumToKorean.prototype.getKorean = function () {
            if (this.formatOption === FormatOptions.SPACING) {
                return this.getSpacing();
            }
            if (this.formatOption === FormatOptions.MIXED) {
                return this.getMixed();
            }
            if (this.formatOption === FormatOptions.LINGUAL) {
                return this.getLingual();
            }
            if (this.formatOption === FormatOptions.LINGUAL_SPACING) {
                return this.getLingualSpacing();
            }
            if (this.formatOption === FormatOptions.NORMAL) {
                return this.getNormal();
            }
            if (this.formatOption === FormatOptions.FLOAT) {
                return this.getFloat();
            }
            return this.getNormal();
        };
        return NumToKorean;
    }());
    /**
     * Converts a number to Korean notation.
     * @param num A number to convert into Korean notation.
     * @param formatOptions A string to select a format.
     */
    function numToKorean(num, formatOptions) {
        var k = new NumToKorean(num, formatOptions || FormatOptions.NORMAL);
        return k.getKorean();
    }

    exports.FormatOptions = FormatOptions;
    exports.numToKorean = numToKorean;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
